#pragma once
#include <iostream>
#include <set>
#include "JsonParser.h"
#include "PGValue.h"

//-----------------------------------------------------------------------------
/**
@brief JSON オブジェクト

JSON のオブジェクト型データを保持します。
*/
class PGJsonObject
{
  public:
	int mDebugFlag;
	void* operator new ( size_t size );

	PGJsonObject(PGJsonObject* parent);
	virtual ~PGJsonObject();

	// 既に仮想化を利用しているもの
	virtual bool HasGroup(); // const
	virtual bool AddObject( PGJsonObject *jo, int command=0 );
	virtual ostream &OutJson( ostream &out ); // const
	virtual ostream &OutJson( ostream &out, int &b ); // const
	virtual ostream &OutJsonMember( ostream &out, int &b ); // const
	virtual ostream &OutJsonDirtyMember( ostream &out, int &b, const SGString& sessionid );
	virtual ostream &OutJsonList( ostream &out, int &b ); // const
	virtual PGJsonObject* SearchName( const SGString &name ); // const
	virtual PGJsonObject* SetValue( const SGString &key, PGValue value, int dirty=-1, const SGString& sessionid="");

	// まだ仮想化として利用されていない
	virtual bool Parse( JsonParser &jp, bool shard );
	virtual SGString GetJson(); // const
	virtual PGJsonObject* SetValueByName( const SGString &key, PGValue value, const SGString &name, int dirty=-1, const SGString& sessionid="");
	virtual PGValue& GetValue( const SGString &key ); // const
	virtual PGJsonObject* GetParent();
	virtual bool HasKey( const SGString &key ) const;

	void Delete(const SGString &point );

	void SetDirty(const SGString &key );
	virtual void ResetDirty();
	bool IsDirty();

	bool IsDirtyAdd();
	bool IsDirtyDel();
	void SetDirtyAdd();
	virtual void SetDirtyDel(const SGString& name);
	const SGString& GetDirtyDel();
	void ResetDirtyAdd();
	void ResetDirtyDel();

	typedef vector<PGJsonObject*> JObjectList;                 ///< @brief JSON オブジェクトリスト型
	typedef vector<PGJsonObject*>::iterator JOIterator;   ///< @brief @ref List のイテレーター型
//	typedef map<SGString,PGValue> KeyValueList;         ///< @brief キー値リスト型
//	typedef map<SGString,PGValue>::iterator KVIterator; ///< @brief @ref KeyValueList のイテレーター型
	typedef set<SGString> DirtyList;         ///< @brief キー値リスト型
	typedef set<SGString>::iterator DLIterator; ///< @brief @ref KeyValueList のイテレーター型

	operator const PGJObject&() const;
	PGJObject& GetObject();

	PGJOIterator KeyValueBegin();
	PGJOIterator KeyValueEnd();

	SGString mName;     ///< @brief 名前
private:
	PGJsonObject* mParent;
	DirtyList mDirty;        ///< @brief ダーティフラグ
	bool mDirtyAdd;
	SGString mDirtyDel;
//	KeyValueList mList; ///< @brief リスト
	PGJObject mList;
};
//-----------------------------------------------------------------------------
/**
@brief ラジオボタン オブジェクト
*/
class PGJsonRadioButtonObject : public PGJsonObject
{
  public:
	PGJsonRadioButtonObject(PGJsonObject* parent );
	virtual ~PGJsonRadioButtonObject();

	virtual PGJsonObject* SetValue( const SGString &key, PGValue value, int dirty=-1, const SGString& sessionid="");
};


//-----------------------------------------------------------------------------
/**
@brief リスト付 JSON オブジェクト
*/
class PGJsonObjectHasList : public PGJsonObject
{
  public:
	PGJsonObjectHasList(const char* listkey, PGJsonObject* parent );
	virtual ~PGJsonObjectHasList();

	void DeleteChildAll();
	void DeleteChild(const SGString& name);

	virtual void ResetDirty();

	virtual bool HasGroup();
	virtual bool AddObject( PGJsonObject *jo, int command=0);
	virtual ostream &OutJsonList( ostream &out, int &b );
	virtual PGJsonObject* SearchName( const SGString &name );
	virtual void SetDirtyDel(const SGString& name);

	SGString mListKey; ///< @brief リストのキー名
	JObjectList mObjectList;  ///< @brief 子 JSON オブジェクトリスト
};

//-----------------------------------------------------------------------------
/**
@brief ラジオボタン グループオブジェクト
*/
class PGJsonRadioButtonGroup : public PGJsonObjectHasList
{
  public:
	PGJsonRadioButtonGroup(PGJsonObject* parent );
	virtual ~PGJsonRadioButtonGroup();

	void SwitchOff();
};
