#pragma once

#include "SGSemaphore.h"

#include <vector>
#include <list>

#include <stdio.h>
#include <iostream>
#include <fstream>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include <string>

//#include "sgstring.h"

using namespace std;

#define SG_SELECT_PIPE_ID  (0x7fffffff)
#define SG_SELECT_START_UNIQUE_ID (0x10000000)
//-----------------------------------------------------------------------------
// SGFdData
//-----------------------------------------------------------------------------
/**
@brief デバイスのファイルディスクリプタを保持するクラス

デバイスのファイルディスクリプタを保持するクラスです。\n
FDをPlusGライブラリで管理させる場合に使用します。\n
使用する場合は、このクラスを基本クラスにしたクラス（SGFdDataから派生したクラス）を
新規に作成して下さい。\n
virtualで用意された２つの関数、ReadyToRead() と ReadyToWrite() をオーバーライドすることで
FDに対する、受信可能（データが到着した）、送信可能（送信できる状態になった）のタイミングを
知ることができるようになります。\n
*/
#define SG_FDTYPE_UNSET  -1
#define SG_FDTYPE_LEGACY  0
#define SG_FDTYPE_FILE    1
#define SG_FDTYPE_SOCKET  2
#define SG_FDTYPE_SOCKET_CONNECT 3
#define SG_FDTYPE_SERIAL  4

#define SG_FDSTATUS_CREATE  0
#define SG_FDSTATUS_OPEN    1
#define SG_FDSTATUS_CONNECT 2
#define SG_FDSTATUS_CLOSE   3
#define SG_FDSTATUS_DELETE  4

#define RECEIVESIZE			1500
#define BUFFERSIZEMIN		(RECEIVESIZE*2)

#define NBUFFEROVERFLOW		1000	// 受信バッファが一杯になりました
#define NCANRECEIVE			1001	// 受信できるようになりました（受信バッファ一杯からの復帰）
#define NREADERROR			1002
#define NCREARBUFFER		1004
class SGFdDataList;
class SGFdData
{
public:
	SGFdData(SGFdDataList* list);
	SGFdData(int type,int fd,int id,SGFdDataList* list);
	virtual ~SGFdData();

	virtual void ReadyToRead();
	virtual void ReadyToWrite();
	virtual int OnReceive();
	virtual int OnNotice(int code,int eno=0);

//	void SetList(SGFdDataList* list);
	SGFdDataList* GetFdDataList();
	int AddEvent();
	void RemoveEvent();

	int GetStatus();
	int GetFDStatus();
	int SetFDStatus(int status);
	int GetFD();
	int SetFD(int fd);
	int GetId();
	void SetId(int id);
	int GetType();
	void SetType(int type);
	int IsSelect();
	void SetSelect(int flag);

	int GetAutoDelete();
	void SetAutoDelete(int enable=1);

	int CreateBuffer(int buffersize);
	int SetDataToBuffer(char* data,int len);
	int GetDataLength();
	int GetData(void* data,int len);
	int CheckBuffer(int notice=1);
	void ClearData();
private:
	int mStatus;     ///< @brief ステータス
	int mType;       ///< @brief ディスクリプタの種類
	int mId;         ///< @brief オブジェクトID
	int mFD;         ///< @brief ファイルディスクリプタ
	int mAutoDelete; ///< @brief 自動解放フラグ
	int mIsSelect;   

protected:
	SGFdDataList* mList;      ///< @brief ファイルディスクリプタリスト
	SGSemaphore mSemData;     ///< @brief バッファ操作セマフォ
	char* mDataBuffer;        ///< @brief バッファ
	volatile int mBufferSize; ///< @brief バッファサイズ
	volatile int mBufferPos;  ///< @brief データ追記位置
};

//-----------------------------------------------------------------------------
// SGFdDataList
//-----------------------------------------------------------------------------
typedef list<SGFdData*> FdDataList;                 ///< @brief ファイルディスクリプタリスト型
typedef FdDataList::iterator FdDLiterator;          ///< @brief @ref FdDataList のイテレーター型
typedef FdDataList::reverse_iterator FdDLriterator; ///< @brief @ref FdDataList の逆方向イテレーター型
/**
@brief デバイスのファイルディスクリプタを保持するクラス（SGFdData）を管理するクラス

デバイスのファイルディスクリプタを保持するクラス（SGFdData）を管理するリスト構造のクラスです。\n
このクラスは SGApplication のメンバとして存在し、ライブラリが使用します。\n
このクラスを直接使用することはありません。
*/
class SGFdDataList : public FdDataList
{
public:
	SGFdDataList();
	virtual ~SGFdDataList();

	void WakeUp();

	int CreateUniqueId();
	int Add(int fd,int id=-1);
	int Add(SGFdData* fdd);
	void Remove(int fd,int id=-1);
	void Remove(SGFdData* fdd);

	int FdSet(fd_set* rfds,fd_set* wfds);
	void FdIsSet(fd_set* rfds,fd_set* wfds,FdDataList* list);

private:
	int mPipeFd;			///< @brief (未記入)
	volatile int mRequest;
	int mUniqueId;
	SGSemaphore mSemFdList; ///< @brief リスト操作セマフォ
	FdDataList mAddRequest;
};
typedef SGFdDataList::iterator         SGFdDLiterator;  ///< @brief @ref SGFdDataList のイテレーター型
typedef SGFdDataList::reverse_iterator SGFdDLriterator; ///< @brief @ref SGFdDataList の逆方向イテレーター型
